@@ -23,10 +23,13 @@ OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 SOFTWARE.
 """
 
-from typing import Iterable, Tuple, Dict, Hashable, List
+from typing import Iterable, Tuple, Dict, Hashable, List, Optional
 import numpy as np
 import math
 import logging
+import warnings
+
+from smqtk_image_io import AxisAlignedBoundingBox
 
 try:
     import torch  # type: ignore
@@ -40,7 +43,6 @@ except ModuleNotFoundError:
     pass
 
 from smqtk_detection import DetectImageObjects
-from smqtk_detection.utils.bbox import AxisAlignedBoundingBox
 
 from importlib.util import find_spec
 deps = ['torch', 'cv2', 'numba']
@@ -66,7 +68,7 @@ class CenterNetVisdrone(DetectImageObjects):
         scales: List[float] = None,
         flip: bool = False,
         nms: bool = True,
-        use_cuda: bool = True,
+        use_cuda: bool = False,
         batch_size: int = 1,
         num_workers: int = 0,
     ):
@@ -97,7 +99,7 @@ class CenterNetVisdrone(DetectImageObjects):
         :param num_workers: Number of subprocesses to use for data loading.
 
         """
-        model_urls = {
+        self.model_urls = {
             "resnet18": "https://download.pytorch.org/models/resnet18-5c106cde.pth",
             "resnet34": "https://download.pytorch.org/models/resnet34-333f7ec4.pth",
             "resnet50": "https://download.pytorch.org/models/resnet50-19c8e357.pth",
@@ -106,7 +108,7 @@ class CenterNetVisdrone(DetectImageObjects):
             "res2net50": "https://shanghuagao.oss-cn-beijing.aliyuncs.com/res2net/res2net50_v1b_26w_4s-3cf99910.pth",
             "res2net101": "https://shanghuagao.oss-cn-beijing.aliyuncs.com/res2net/res2net101_v1b_26w_4s-0812c246.pth",
         }
-        resnet_spec = {
+        self.resnet_spec = {
             "resnet18": (_ResNet, _BasicBlock_ResNet, [2, 2, 2, 2]),
             "resnet34": (_ResNet, _BasicBlock_ResNet, [3, 4, 6, 3]),
             "resnet50": (_ResNet, _Bottleneck_ResNet, [3, 4, 6, 3]),
@@ -115,8 +117,9 @@ class CenterNetVisdrone(DetectImageObjects):
             "res2net50": (_Res2Net, _Bottleneck_Res2Net, [3, 4, 6, 3]),
             "res2net101": (_Res2Net, _Bottleneck_Res2Net, [3, 4, 23, 3]),
         }
-
-        heads = {'hm': 10, 'wh': 2, 'reg': 2}   # type: Dict[Hashable, int]
+        if arch not in self.model_urls:
+            raise ValueError(f"Invalid architecture provided. Must be one of: "
+                             f"{list(self.model_urls)}")
 
         self.arch = arch
         self.model_file = model_file
@@ -140,6 +143,8 @@ class CenterNetVisdrone(DetectImageObjects):
         self.out_w = self.input_w // 2
 
         self.num_classes = 10
+        assert self.num_classes == len(CLASS_NAMES), \
+            "Mismatch in num_classes record and actual list of class names."
 
         # visdrone2019 mean and standard deviation, used for normalization
         self.mean = np.asarray([[[0.372949, 0.37837514, 0.36463863]]], dtype=np.float32)
@@ -154,11 +159,20 @@ class CenterNetVisdrone(DetectImageObjects):
         else:
             self.device = torch.device('cpu')
 
-        self.model = _CenterNet(resnet_spec[self.arch], heads)
-        self.model.init_weights(model_urls[self.arch], pretrained=True)
-        self.model = self.model.to(self.device)
-        self.model = _load_model(self.model, model_file)
-        self.model = self.model.eval()
+        self.model: Optional[_CenterNet] = None
+
+    def _get_model(self) -> "_CenterNet":
+        """
+        Lazy initialize the model. Idempotent.
+        """
+        if self.model is None:
+            heads = {'hm': 10, 'wh': 2, 'reg': 2}   # type: Dict[Hashable, int]
+            model = _CenterNet(self.resnet_spec[self.arch], heads)
+            model.init_weights(self.model_urls[self.arch], pretrained=True)
+            model = model.to(self.device)
+            self.model = model = _load_model(model, self.model_file)
+            model.eval()
+        return self.model
 
     def detect_objects(
         self,
@@ -167,6 +181,7 @@ class CenterNetVisdrone(DetectImageObjects):
 
         img_set = _ImageDataset(img_iter, self.flip, self.scales, self._preprocess_img)
         batch_loader = DataLoader(img_set, batch_size=self.batch_size, num_workers=self.num_workers)
+        model = self._get_model()
 
         # need to match flipped batched outputs if using flip and batch size is odd
         align_needed = self.flip and self.batch_size % 2 != 0
@@ -186,7 +201,7 @@ class CenterNetVisdrone(DetectImageObjects):
             trans_scales = trans_scales.cpu().numpy()
 
             with torch.no_grad():
-                output = self.model(img_tensor)
+                output = model(img_tensor)
 
             score_maps = output['hm'].sigmoid_()  # detection scores
             size_maps = output['wh']  # bbox sizes
@@ -337,20 +352,17 @@ class CenterNetVisdrone(DetectImageObjects):
         Converts detection matrix to the format required by the
         ``DetectImageObjects`` interface.
         """
-
         # empty dict to fill
-        # applicable visdrone classes start at 1
-        zero_dict = {i: 0 for i in range(1, self.num_classes+1)}  # type: Dict[Hashable, float]
-
+        zero_dict: Dict[Hashable, float] = {lbl: 0. for lbl in CLASS_NAMES}
+        # batch round and cast index column.
+        class_indices = dets_mat[:, 5].round().astype(int)
+        # Create and collect tuples.
         dets_list = []
-        for det in dets_mat:
+        for det, cls_idx in zip(dets_mat, class_indices):
             bbox = AxisAlignedBoundingBox(det[0:2], det[2:4])
-
             class_dict = zero_dict.copy()
-            class_dict[det[5] + 1] = det[4]
-
+            class_dict[CLASS_NAMES[cls_idx]] = det[4]
             dets_list.append((bbox, class_dict))
-
         return dets_list
 
     def get_config(self) -> dict:
@@ -358,18 +370,29 @@ class CenterNetVisdrone(DetectImageObjects):
             "arch": self.arch,
             "model_file": self.model_file,
             "max_dets": self.max_dets,
-            "K": self.k,
+            "k": self.k,
             "scales": self.scales,
             "flip": self.flip,
-            "use_cuda": self.use_cuda
+            "nms": self.nms,
+            "use_cuda": self.use_cuda,
+            "batch_size": self.batch_size,
+            "num_workers": self.num_workers,
         }
 
     @classmethod
     def is_usable(cls) -> bool:
+        if not usable:
+            warnings.warn(
+                f"CenterNetVisdrone is not usable. Dep status: "
+                f"{ {k: (v is not None) for k, v in zip(deps, specs)} }",
+                RuntimeWarning,
+            )
         return usable
 
 
 if usable:
+    CLASS_NAMES = ['pedestrian', 'people', 'bicycle', 'car', 'van', 'truck',
+                   'tricycle', 'awning-tricycle', 'bus', 'motor']
 
     class _ImageDataset(Dataset):
         """
@@ -462,7 +485,7 @@ if usable:
         model.load_state_dict(state_dict, strict=False)
 
         # resume optimizer parameters
-        if optimizer is not None:
+        if optimizer is not None:  # not used in smqtk use-case  # pragma: no cover
             if "optimizer" in checkpoint:
                 optimizer.load_state_dict(checkpoint["optimizer"])
                 start_epoch = checkpoint["epoch"]
@@ -476,7 +499,7 @@ if usable:
                 logger.info("Resumed optimizer with start lr", start_lr)
             else:
                 logger.info("No optimizer parameters in checkpoint.")
-        if optimizer is not None:
+        if optimizer is not None:  # not used in smqtk use-case  # pragma: no cover
             return model, optimizer, start_epoch
         else:
             return model
@@ -626,7 +649,7 @@ if usable:
         Nt=0.3,
         threshold=0.001,
         method=0
-    ):
+    ):  # pragma: no cover
         N = boxes.shape[0]
         pos = 0
         maxscore = 0
@@ -864,7 +887,7 @@ if usable:
                                     nn.init.normal_(m.weight, std=0.001)
                                     nn.init.constant_(m.bias, 0)
 
-                pretrained_state_dict = model_zoo.load_url(url)
+                pretrained_state_dict = model_zoo.load_url(url, map_location='cpu')
                 logger.info("=> loading pretrained model {}".format(url))
                 self.load_state_dict(pretrained_state_dict, strict=False)
 
